from signalrcore_async.hub_connection_builder import HubConnectionBuilder
#from signalrcore_async.protocol.msgpack import MessagePackHubProtocol
import logging
from urllib.parse import quote



class ApiData():
    """
    Provides methods and properties to connect, stream and get market data
    """

    def __init__(self):
        self.connection_started = None
        self.connection_stopped = None
        self.on_trade = None
        self.on_best = None
        self.on_refs = None
        self.on_srefs = None

    async def initialize(self, api_token, api_host,scheme):
        """
        initialize the api with token for authentication and authorization
        """
        api_token = quote(api_token)
        print(api_token)
        protocol = scheme
        host = api_host
        # host = "localhost:5011"
        hub_url = f"{protocol}://{host}/api/fda/apidata/stream?api_token={api_token}"
        global connection
        connection = HubConnectionBuilder()\
            .with_url(hub_url)\
            .build()
        connection.on_close(self.connection_stopped)
        connection.on_open(self.connection_started)

        try:
            await connection.start()
            print("connection estabilished")

            connection.on("_t", self.on_trade)
            connection.on("t", self.on_trade)
            connection.on("_b", self.on_best)
            connection.on("b", self.on_best)
            connection.on("f", self.on_refs)
            connection.on("_r", self.on_srefs)
            
            
            return "ok"
        except Exception as err:
            print(err)
            return "not_ok"

    def on_connection_started(self, callback):
        self.connection_started = callback

    def on_connection_stopped(self, callback):
        self.connection_stopped = callback

    # region callbacks

    def on_trade_update(self, callback):
        """
        Tick data update.
        Returns apidata_models.Trade as json
        """
        self.on_trade = callback

    def on_best_update(self, callback):
        self.on_best = callback
        
    def on_refs_update(self, callback):
        self.on_refs = callback

    def on_srefs_update(self, callback):
        self.on_srefs = callback

    # endregion
    
    #region stream subscription methods

    async def subscribeTrade(self, syms):
        await connection.invoke("Ticks", [syms])

    async def subscribeBestAndRefs(self, syms):
        await connection.invoke("Others", [syms])

    async def subscribeAll(self, syms):
        await connection.invoke("SubscribeAll", [syms])

    async def unsubscribeAll(self, syms):
        await connection.invoke("UnsubscribeAll", [syms])

    #endregion

    #region eod data -start
    async def get_eod(self, ticker, startDate, endDate):
        """
        @param : ticker , startDate , EndDate
        ticker : BANKNIFTY-1 , NIFTY-1 , TCS
        startDate and endDate : formate : yyyyMMdd eg:20201001

        eg : ['NIFTY-1','20200828', '20200901']
        """
        if((not ticker) or (startDate.isdigit() == False) or (len(startDate) != 8) or (len(endDate) != 8) or (endDate.isdigit() == False)):
            print("not enough arguments")
            return

        ed = await connection.invoke("Eod", [ticker, startDate, endDate])
        return ed

    async def get_eod_contract(self, underlyingTicker, startDate, endDate, contractExpiry):
        """
        @param : ticker , startDate , EndDate , contractExpiry
        underlyingTicker : BANKNIFTY , NIFTY
        startDate and endDate : formate : yyyyMMdd eg:20201001
        contractExpiry : formate : yyyyMMdd eg: 20201126

        eg: ['NIFTY', '20200828', '20200901', '20201029']
        
        """
        
        if((not underlyingTicker) or (startDate.isdigit() == False) or (len(startDate) != 8 ) or (endDate.isdigit() == False) or (len(endDate) != 8) or (contractExpiry.isdigit() == False) or (len(contractExpiry) != 8)):
            print("not enough arguments")
            return

        ed = await connection.invoke("EodContract", [underlyingTicker, startDate, endDate, contractExpiry])
        return ed

    async def get_intra_eod(self, ticker = "" , startDate = "", endDate = "", resolution="5"):
        """
        @param : ticker , startDate , EndDate , resolution
        ticker : BANKNIFTY-1 , NIFTY-1 , TCS
        startDate and endDate : formate : yyyyMMdd eg:20201001
        resolution : time resolution in minutes. default: 5 minutes

        eg : ['NIFTY-1', '20200828', '20200901', '5']
        """
        if((not ticker) or (startDate.isdigit() == False) or (len(startDate) !=8) or (endDate.isdigit() == False) or (len(endDate) != 8) or (not resolution)):
            print("not enough arguments")
            return

        ied = await connection.invoke("IEod", [ticker, startDate, endDate, resolution])
        return ied

    async def get_intra_eod_contract(self , underlyingTicker = "" , startDate = "" , endDate = "" , contractExpiry = "" , resolution = "5"):
        """
        @param : ticker , startDate , EndDate , contractExpiry , resolution

        ticker : BANKNIFTY-1 , NIFTY-1 , TCS
        startDate and endDate : formate : yyyyMMdd eg:20201001
        contractExpiry : formate : yyyyMMdd eg: 20201126
        resolution : time resolution in minutes. default: 5 minutes

        eg:['NIFTY', '20200828', '20200901', '20201029', '5']
        """
        if((not underlyingTicker) or (startDate.isdigit() == False) or (len(startDate) != 8) or (endDate.isdigit() == False) or (len(endDate) != 8) or (not resolution)):
            print("not enough arguments")
            return

        ied = await connection.invoke("IEodContract", [underlyingTicker, startDate, endDate, contractExpiry, resolution])
        return ied

    async def get_back_ticks(self,ticker,lastDateTime):
        
        if((not ticker) or (not lastDateTime)):
            print("not enough arguments")
            return

        ied = await connection.invoke("TicksHistory", [ticker,lastDateTime])
        return ied
        
    # endregion



import pathlib
from setuptools import setup ,find_packages
from os import path
# The directory containing this file
#HERE = pathlib.Path(__file__).parent

# The text of the README file
#README = (HERE / "README.md").read_text()
HERE = path.abspath(path.dirname(__file__))
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    README = f.read()

# This call to setup() does all the work
setup(
    name="pix_apidata",
    version="1.2.9",
    author="Coumar Pandourangane",
    author_email="pscoumar@gmail.com",
    description="Python library to connect and stream the market data.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/pscoumar",
    license="MIT",
    classifiers=[ 
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    keywords = 'stock market, apidata, accelpix, ticanalytics',
    packages=['pix_apidata'],
    include_package_data=True,
    install_requires=['signalrcore-async', 'urllib3'],

)



import asyncio
import apidata_lib
import apidata_models
import time
#from pix_apidata import apidata_lib,apidata_models

api = apidata_lib.ApiData()

event_loop = asyncio.get_event_loop()


async def main():
    api.on_connection_started(connection_started)
    api.on_connection_stopped(connection_stopped)
    api.on_trade_update(on_trade)
    api.on_best_update(on_best)
    api.on_refs_update(on_refs)
    api.on_srefs_update(on_srefs)
    
    key = "l+phpXGgBZx/g3t+F0ZpW6xNJEA="
    host = "apidata.accelpix.in"
    #key="OMLW74kiRTZeuBIWiJ4gXKZPsPk="
    #host ="192.168.56.101"
    # host = "localhost:5011"
    s = await api.initialize(key, host,"http")
    print(s)
    
    #history data
    his1 = await api.get_eod('TCS', '20200828', '20200901')
    print("History : ",his1)

    master = await api.getMaster()
    print(master)
    #end

    #Live data
    syms = ["TCS",'NIFTY-1']
    await api.subscribeAll(syms)
    time.sleep(1)
    await api.unsubscribeAll(['TCS'])
    print("=====Unsubscribe Done=====")
    

def connection_started():
    print("Connection started callback")


def connection_stopped():
    print("Connection stopped callback")

def on_trade(msg):
    t = apidata_models.Trade(msg)
    print("Trade :",t.ticker)#trade

def on_best(msg):
    b = apidata_models.Best(msg)
    print("best :",msg)#best

def on_refs(msg):
    ref = apidata_models.Refs(msg)
    print("refs",msg)#refs

def on_srefs(msg):
    sref = apidata_models.RefsSnapshot(msg)
    print("sref:",msg)
#end

event_loop.create_task(main())
event_loop.run_forever()



# modules
import json


class Master:
    def __init__(self, json_msg):
        for i in json_msg:
            jsonStr = json.dumps(i)
            parse = json.loads(jsonStr)
        self.xid = parse["xid"]
        self.tkr = parse["tkr"]
        self.atkr = parse["atkr"]
        self.ctkr = parse["ctkr"]
        self.exp = parse["exp"]
        self.utkr = parse["utkr"]
        self.inst = parse["inst"]
        self.a3tkr = parse["a3tkr"]
        self.sp = parse["sp"]
        self.tk = parse["tk"]
        #self.time = parse["time"]


class Trade:
    def __init__(self, json_msg):
        for i in json_msg:
            jsonStr = json.dumps(i)
            parse = json.loads(jsonStr)
        self.id = parse["id"]
        self.kind = parse["kind"]
        self.ticker = parse["ticker"]
        self.segmentId = parse["segmentId"]
        self.price = parse["price"]
        self.qty = parse["qty"]
        self.volume = parse["volume"]
        self.oi = parse["oi"]
        #self.time = parse["time"]


class Best:
    def __init__(self, json_msg):  # Result
        for i in json_msg:
            jsonStr = json.dumps(i)
            parse = json.loads(jsonStr)
        self.ticker = parse["ticker"]  # selff.ticker=NIFTY-1
        self.segmentId = parse["segmentId"]  # 2
        self.kind = parse["kind"]  # B
        self.bidPrice = parse["bidPrice"]  # 13135.5
        self.bidQty = parse["bidQty"]  # 300
        self.askPrice = parse["askPrice"]  # 29865.25
        self.askQty = parse["askQty"]  # 50
        # self.time


class RefsSnapshot:
    def __init__(self, json_msg):
        for i in json_msg:
            jStr = json.dumps(i)
            parse = json.loads(jStr)
        self.kind = parse["kind"]
        self.ticker = parse["ticker"]
        self.segmentId = parse["segmentId"]
        self.open = parse["open"]
        self.close = parse["close"]
        self.high = parse["high"]
        self.low = parse["low"]
        self.avg = parse["avg"]
        self.oi = parse["oi"]


class Refs:
    def __init__(self, json_msg):
        for i in json_msg:
            jStr = json.dumps(i)
            parse = json.loads(jStr)
        self.kind = parse["kind"]
        self.ticker = parse["ticker"]
        self.segmentId = parse["segmentId"]
        self.price = parse["price"]


# End Module
